rm(list=ls())
setwd("/Users/BSS/Documents/7.Research/Cluster_Stability_Final")

source("A0_Functions.R")

########################################################
# 1. Data generation and Alignment
########################################################

dat = list()
K=list()
zb.k = list()
id.k = list()
repre.k = list()
#M=list()
#zb.m = list()
#id.m = list()
#repre.m = list()

for(it in 1:5){
  ########################################################
  #### Generate Data
  ########################################################
  dat[[it]] = GenData2(n=5000,p=20,C=8,k=4)
  #draw.obs(dat$X,dat$z)
  
  ########################################################
  #### Draw whole sample plots
  ########################################################
  C=8;G=8
  X = dat[[it]]$X
  #set.seed(21)
  K[[it]] = kclst(X,X)
  #M = mclst(X,X)
  
  ########################################################
  #### Generate Bootstrap samples
  ########################################################
  
  zb.k[[it]] = GenBsSamps(X,100,kclst)
  #zb.m = GenBsSamps(X,100,mclst)
  
  #save(zb.k,zb.m, file="zbdata.Rdata")
  #load("zbdata.Rdata")
  
  ########################################################
  #### Alignment
  ########################################################
  
  id.k[[it]] = align2(zb.k[[it]]$Z,100,folder="Kcls_jan08")
  #id.m = align2(zb.m$Z,100,folder="Mcls_jan08")
  ########################################################
  #### Find representative
  ########################################################
  
  repre.k[[it]] = Repre2(zb.k[[it]]$Z, 100, id.k[[it]]$idx)
  #repre.m = Repre2(zb.m$Z, 100, id.m$idx)
  
  iK = K[[it]]
  irepre.k = repre.k[[it]]
  iid.k = id.k[[it]]
  izb.k = zb.k[[it]]
  
  save(iK, izb.k, iid.k, irepre.k,
       file = paste("kclt_r1000_",it,".RD",sep=""))
}

########################################################
# 2. Other Methods
########################################################

#= CSPA  for K-means ======================
setwd("kclst")

a = list()
a[[1]] = paste("R",1:1000,sep="")
a[[2]] = paste("C",1:100,sep="")
a[[4]] = 8

cspa=list()

for(j in 1:1000){
  raw = zb[[j]]$Z
  dat = array(NA,dim=c(1000,100,1,1))
  
  for(i in 1:100){
    dat[,i,1,1] = raw[((i-1)*1000+1):(i*1000)]
  }
  dimnames(dat) = a
  cspa[[j]] = CSPA(dat, k = 8)
  print(j)
}

save(cspa,file="repeat1000_K_cspa.RD")

setwd("..")

#= CSPA  for Mclust ======================
setwd("mclst")

a = list()
a[[1]] = paste("R",1:1000,sep="")
a[[2]] = paste("C",1:100,sep="")
a[[4]] = 8

cspa.m=list()

for(j in 1:1000){
  raw = zb[[j]]$Z
  dat = array(NA,dim=c(1000,100,1,1))
  
  for(i in 1:100){
    dat[,i,1,1] = raw[((i-1)*1000+1):(i*1000)]
  }
  dimnames(dat) = a
  cspa.m[[j]] = CSPA(dat, k = 8)
  print(j)
}

save(cspa.m,file="repeat1000_M_cspa.RD")

setwd("..")

#= Co-Association matrix  =========================

lCA = list()
n=1000;nb=100
for(i in 1:1000){
  load(paste("kclt_r1000_",i,".RD",sep=""))
  P = list()
  for(t in 1:100){
    P[[t]] = izb.k$Z[(1+(t-1)*n):(t*n)]
  }
  
  CA = matrix(NA,1000,1000)
  T=1:100
  CA[n,n] = 1
  for(x in 1:(n-1)){
    CA[x,x] = 1
    for(y in (x+1):n){
      CA[x,y] = 1/nb*sum(as.numeric(lapply(T, function(t) 1*(P[[t]][x]==P[[t]][y]))))
    }
  }
  lCA[[i]] = CA
  save(CA, file=paste("CAmx_K_",i,".RD",sep=""))
}


########################################################
# 3. Analysis
########################################################

setwd("kclst")

K = list()
repre.k = list()
id = list()
zb = list()

for(i in 1:1000){
  load(paste("kclt_r1000_",i,".RD",sep=""))
  K[[i]] = iK
  repre.k[[i]] = irepre.k
  id[[i]] = iid.k
  zb[[i]] = izb.k
  print(i)
}

#save(zb,file="repeat1000_K_zb.RD")
#save(dat,K,repre.k,id,file="repeat1000_K.RD")
#load("repeat1000_K_zb.RD")
#load("repeat1000_K.RD")
setwd("..")

#= median cls (CSPA) =============================

draw.obs5(dat[[i]]$X,cspa[[i]])

#= median cls (Wasserstein) ======================
n = 1000

med.W = list()
idx.W = rep(NA,1000)
for(i in 1:1000){
  idx.W[i] = as.numeric(id[[i]])
  med.W[[i]] = zb[[i]]$Z[(1+(idx.W[i]-1)*n):(idx.W[i]*n)]
}
idx.W
draw.obs5(dat[[i]]$X,med.W[[i]])

#= median cls (Adj Rand Ind) =====================
n = 1000
#i=1;j=1;k=2
ARI = list()
for(i in 1:1000){
  ARI[[i]] = matrix(NA,100,100)
  ARI[[i]][100,100] = 1
  for(j in 1:99){
    ARI[[i]][j,j] = 1
    for(k in (j+1):100){
      ARI[[i]][j,k] = adjustedRandIndex(zb[[i]]$Z[(1+(j-1)*n):(j*n)],
                                        zb[[i]]$Z[(1+(k-1)*n):(k*n)])
      ARI[[i]][k,j] = ARI[[i]][j,k]
    }
    print(i+j/100)
  }
}

#save(ARI,file="ARImx_K.RD")
load("ARImx_K.RD")

med.R = list()
idx.R = rep(NA,1000)
for(i in 1:1000){
  idx.R[i] = which.max(rowMeans(ARI[[i]]))
  med.R[[i]] = zb[[i]]$Z[(1+(idx.R[i]-1)*n):(idx.R[i]*n)]
}

idx.R
draw.obs5(dat[[i]]$X,med.R[[i]])

sum(idx.W == idx.R)

#= Co-Association matrix =========================

lCA = list()
lERR = list()
setwd("kclst/CA_k")
for(i in 1:1000){
  load(paste("CAmx_K_",i,".RD",sep=""))
  MVCA = 1*(CA>0.5)
  CAZ = rep(NA,1000)
  CAZ[1] = 1
  k=1
  #ERR = rep(0,1000)
  #ERR[1] = 1
  for(j in 1:1000){
    k=max(CAZ,na.rm=T)+1
    CAZ[which(MVCA[j,]==1)] = ifelse(is.na(CAZ[j]),k,CAZ[j])
    #l=k/(10^k)
    #ERR[which(MVCA[j,]==1)] = ifelse(ERR[j]==0,ERR[which(MVCA[j,]==1)]+l,ERR[which(MVCA[j,]==1)])
    
  }
  lCA[[i]] = CAZ
  #lERR[[i]] = ERR
  print(i)
}
#lapply(lERR,table)
setwd("..")


#save(lCA, file="repeat_CAmx_1000_K.RD")
load("repeat_CAmx_1000_K.RD")

setwd("..")

i=100
draw.obs5(dat[[i]]$X,lCA[[i]])

#=================================================
l.n.dist = list()
n=1000
for(i in 1:1000){
  n.zb = c(dat[[i]]$z,
           K[[i]],
           med.R[[i]],
           med.W[[i]],
           cspa[[i]],
           lCA[[i]],
           repre.k[[i]]$repre)-1
  setwd("../package5")
  write(n.zb, file = "zb.cls", ncolumns = 1)
  system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",length(n.zb)/n," -t ",0.8," -2",sep=""))
  n.dist  <- read.table("zb.par")
  setwd(wd)
  
  l.n.dist[[i]] = n.dist
}

#save(l.n.dist, file="l.n.dist.Rd")
d.dist = matrix(NA,1000,7)
m.dist = matrix(NA,1000,7)
r.dist = matrix(NA,1000,7)
for(i in 1:1000){
  m.dist[i,] = l.n.dist[[i]][,1]
  d.dist[i,] = l.n.dist[[i]][,2]
  r.dist[i,] = c(1-adjustedRandIndex(dat[[i]]$z,dat[[i]]$z),
                 1-adjustedRandIndex(dat[[i]]$z,K[[i]]),
                 1-adjustedRandIndex(dat[[i]]$z,med.R[[i]]),
                 1-adjustedRandIndex(dat[[i]]$z,med.W[[i]]),
                 1-adjustedRandIndex(dat[[i]]$z,cspa[[i]]),
                 1-adjustedRandIndex(dat[[i]]$z,lCA[[i]]),
                 1-adjustedRandIndex(dat[[i]]$z,repre.k[[i]]$repre))
}

range(dat[[i]]$z)
range(med.R[[i]])
range(med.W[[i]])
range(cspa[[i]])
range(lCA[[i]])
range(repre.k[[i]]$repre)

m.dist
d.dist
r.dist
A = rbind(apply(m.dist,2,function(x) sum(x==6)),
          apply(m.dist,2,function(x) sum(x==7)),
          apply(m.dist,2,function(x) sum(x==8)))
rownames(A) = 6:8
colnames(A) = 1:3
A/10

a.dist=r.dist
apply(a.dist,2,function(x) sum(x==0))/10
colSums(apply(a.dist[,-c(1,4)],2,function(x) x==rowMin(a.dist[,-c(1,4)])))/10
round(apply(a.dist,2,mean),3)
round(sqrt(apply(a.dist,2,var)),3)

i=sample(1:1000,1)
draw.obs(dat[[i]]$X,dat[[i]]$z)

########################################################
library(ggplot2)
library(dplyr)
draw.obs2 <- function(Y,true_cell_cls){
  X = as.data.frame(prcomp(Y)$x[,1:2])
  K = true_cell_cls
  A = as.character(true_cell_cls)
  
  s = cbind(X,A) %>% split(K)
  ch = s %>% lapply(., function(el) chull(el$PC1, el$PC2))
  ch = lapply(names(ch), function(x) s[[x]][ch[[x]],]) %>% do.call(rbind, .) 
  
  mu_x = sapply(1:max(K),function(z){mean(X[which(K==z),1])})
  mu_y = sapply(1:max(K),function(z){mean(X[which(K==z),2])})
  
  ggplot(as.data.frame(X[,1:2]), aes(x=PC1, y=PC2, color = A)) +
    geom_point(shape=19, size=1, alpha=0.5) +
    geom_polygon(data = ch, aes(fill = A), alpha = 0.2) +
    theme_classic() +
    annotate("text", x=mu_x, y=mu_y, label=1:max(K), size=4, fontface = "bold")
}

########################################################
i=370
draw.obs2(dat[[i]]$X, dat[[i]]$z)
ggsave("E1_original.jpeg",width=100,height=100,units="mm")
draw.obs2(dat[[i]]$X, K[[i]])
ggsave("E1w_kclt.jpeg",width=100,height=100,units="mm")
draw.obs2(dat[[i]]$X, repre.doubl_med[[i]]$repre)
ggsave("E1r_kclt.jpeg",width=100,height=100,units="mm")
