# install required packages
library('devtools')

source("https://bioconductor.org/biocLite.R")
biocLite("pcaMethods")
install_github('JustinaZ/pcaReduce')
install_github('Henrygwb/HMMVB')

library('pcaReduce')
library("HMMVB")
library("mclust")
library("pryr")

########################################################
# 1. Data alignment
########################################################

rm(list=ls())
setwd("/Users/BSS/Documents/7.Research/Cluster_Stability_Final")

source("A0_Functions.R")

zb = list()
zb.align = list()
repre.quick_med = list()
repre.quick_mod = list()
#repre_med = list()
#repre_mod = list()
M = list()
stat = list()

for(i in 1:30){
########################################################
#### Generate Data
########################################################
  b <-read.table("Pollen2014.txt", sep=',', header = T,row.names=1)
  
  lb <-read.table("SupplementaryLabels.txt", sep=',', header = T)
  
  D <- log2(as.matrix(b) + 1) # log transformation of count data
  
  #Input <- t(D) # data matrix, cells in rows, genes in columns
  #-------------------------------------------
  cd <- b
  cd <- cd[, colSums(cd>0)>1.8e3]
  # remove genes that don't have many reads
  cd <- cd[rowSums(cd)>10, ]
  # remove genes that are not seen in a sufficient number of cells
  cd <- cd[rowSums(cd>0)>5, ]
  # transform to make more data normal
  mat <- log10(as.matrix(cd)+1)
  v <- apply(mat, 1, var)
  vi <- names(sort(v, decreasing = T)[1:500])
  Input <- t(mat[vi,]) # data
  #-------------------------------------------
  true_tissue_cls <- lb[,6] # true data partition K=4
  true_cell_cls <- lb[,4]   # true data partition K=11
  true_cell <- lb[,3]

########################################################
#### Draw whole sample plots
########################################################

X = Input           
C=15;G=15;
NVB=5;DSQ=rep(dim(X)[2]/NVB,NVB);CSQ=rep(40,NVB)

#K[[i]] = kclst(X,X)
#M[[i]] = mclst(X,X)
M[[i]] = hmmvb2(X,X)
#draw.obs(X,kclst(X,X))
#draw.obs2(X,M[[i]])

########################################################
#### Generate Bootstrap samples
########################################################

zb[[i]] = GenBsSamps(X,100,hmmvb2)

########################################################
#### Alignment
########################################################

zb.align[[i]] = align(zb[[i]]$Z,100,folder="V_Feb23")

########################################################
#### Find representative
########################################################

repre.quick_med[[i]] = Repre.quick(zb[[i]]$Z, zb.align[[i]]$dist, zb.align[[i]]$wt, method="median")
repre.quick_mod[[i]] = Repre.quick(zb[[i]]$Z, zb.align[[i]]$dist, zb.align[[i]]$wt, method="mode")
#draw.obs2(X,repre.quick_med[[i]]$repre)


########################################################
#### Find statistics
########################################################

stat[[i]] = StatForClst(repre.quick_med[[i]]$repre,zb[[i]]$Z)

iM=M[[i]];izb=zb[[i]];izb.align=zb.align[[i]]
irepre.quick_med=repre.quick_med[[i]];irepre.quick_mod=repre.quick_mod[[i]]
istat=stat[[i]]
save(iM,
     izb,izb.align,
     irepre.quick_med,irepre.quick_mod,
     istat,
     file=paste("vclt_bs_",i,".RD",sep=""))
}

########################################################
# 2. Analysis
########################################################
########################################################
#### check data
########################################################
true_tissue_cls <- lb[,6] # true data partition K=4
true_cell_cls <- lb[,4]   # true data partition K=11
true_cell <- lb[,3]

library(ggplot2)
library(dplyr)
draw.obs2 <- function(X,true_cell_cls, title=""){
  #X = as.data.frame(prcomp(X)$x[,1:2])
  X = as.data.frame(X)
  K = true_cell_cls
  A = as.character(true_cell_cls)
  
  ggplot(as.data.frame(X[,1:2]), aes(x=PC1, y=PC2, color = A)) +
    geom_point(shape=19, size=1, alpha=0.7) +
    theme_classic() +
    ggtitle(title) +
    guides(color=guide_legend(title="Cell Type"))
}

draw.obs3 <- function(X,true_cell_cls, title=""){
  #X = as.data.frame(prcomp(X)$x[,1:2])
  X = as.data.frame(X)
  K = true_cell_cls
  A = as.character(true_cell_cls)
  
  ggplot(as.data.frame(X[,1:2]), aes(x=PC1, y=PC2, color = A)) +
    geom_point(shape=19, size=1, alpha=0.7) +
    theme_classic() +
    theme(axis.title.x=element_blank(),
          axis.text.x=element_blank(),
          axis.title.y=element_blank(),
          axis.text.y=element_blank()) +
    guides(color=FALSE)
}
img.gt = draw.obs2(X2,true_cell)#,title="Ground Truth")
ggsave("RD_GT.jpeg",width=110,height=80,units="mm")

########################################################
#### Draw whole sample plots
########################################################
C=15; G=15
X = Input
newdata = Input
X2 = prcomp(t(D))$x[,1:2]

NVB=5;DSQ=rep(dim(X)[2]/NVB,NVB);CSQ=rep(40,NVB)

load("C0_P.Rdata")
img.p0 = draw.obs4(X2,P11)

load("C0_V.Rdata")
img.v0 = draw.obs4(X2,V)
#ggsave("R1sc-hmmvb.jpeg",width=100,height=100,units="mm")

########################################################
#### Find representative
########################################################

setwd("pcl14")
load("14pclt_bs_2.RD")
setwd("..")
repre.p = irepre.quick_med
stat.p = istat
img.pm = draw.obs4(X2,repre.p$repre)
#ggsave(paste("P4_",iter,".jpeg",sep=""),width=100,height=100,units="mm")

setwd("hmm_bs")
load("vclt_bs_1.RD")
setwd("..")
repre.v = irepre.quick_med
stat.v = istat
img.vm = draw.obs4(X2,repre.v$repre)
#ggsave(paste("P4_",iter,".jpeg",sep=""),width=100,height=100,units="mm")

#------------------------------------------
# for individual cluster study
id.v = align2(izb$Z+1,100,folder="hmmvb03")
repre.v2 = Repre2(izb$Z+1, 100, id.v$idx)
i=1
for(i in (1:9)/10){
  assign(paste("repre.v2.",i,sep=""),
         Repre2(izb$Z+1, 100, id.v$idx, alpha=i))
}
img.vm = draw.obs4(X2,repre.v2$repre)
img.vm

#------------------------------------------
# jaccard mat
draw.obs4(X2,repre.v2$repre)
draw.obs4(X2,ifelse(repre.v2$repre==6,1,0))
draw.obs4(X2,ifelse(true_cell=="iPS",1,0))
unique(repre.v2$repre)

JM = repre.v2$jaccard_mat
JM = JM[-um,-um]
JM*100

SM = repre.v2$summary
SM = SM[-um,c(3,5,7,9)]
t(SM)
#1 HL60
#3 Cl1
#4 2338
#5 K562
#6 NPC
#8 Kera
#11 BJ
#12 Cl2
#13 iPS
#18 2339
#unmatched 
um = c(2,7,9,10,14,15,16,17)

table(repre.v2$confset[[1]]+repre.v2$confset[[13]])
table(repre.v2$confset[[3]]+repre.v2$confset[[13]])
table(repre.v2$confset[[4]]+repre.v2$confset[[13]])
table(repre.v2$confset[[5]]+repre.v2$confset[[13]])
table(repre.v2$confset[[6]]+repre.v2$confset[[13]])
table(repre.v2$confset[[8]]+repre.v2$confset[[13]])
table(repre.v2$confset[[11]]+repre.v2$confset[[13]])
table(repre.v2$confset[[12]]+repre.v2$confset[[13]])
table(repre.v2$confset[[13]]+repre.v2$confset[[13]])
table(repre.v2$confset[[18]]+repre.v2$confset[[13]])

#------------------------------------------
n=300
ta=0.999

n.zb = c(true_cell_cls-1,
         repre.v2$repre-1)
setwd("../_package5")
write(n.zb, file = "zb.cls", ncolumns = 1)
system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",length(n.zb)/n," -t ",ta," -2",sep=""))

wt <- read.table("zb.wt")

setwd(wd)

wt = wt[-um,]
colnames(wt) = unique(true_cell)
wt[c(2,8),]
round(wt/rowSums(wt),3)
#------------------------------------------  

t(repre.v2$summary[c(1,3,4,5,6,8,11,12,13,18),c(3,5,7,9)])
match(which(true_cell=="Kera"),which(repre.v2$repre==8))
match(which(true_cell=="HL60"),which(repre.v2$repre==1))

#draw.obs5(X2,repre.v2$confset[[1]])
#draw.obs5(X2,repre.v2$confset[[3]])
#draw.obs5(X2,repre.v2$confset[[4]])
#draw.obs5(X2,repre.v2$confset[[5]])
#draw.obs5(X2,repre.v2$confset[[6]])
#draw.obs4(X2,repre.v2$confset[[8]])
#draw.obs5(X2,repre.v2$confset[[11]])
#draw.obs5(X2,repre.v2$confset[[12]])
draw.obs4(X2,repre.v2$confset[[13]])
#draw.obs5(X2,repre.v2$confset[[18]])

Ch <- function(X,A){
  s = cbind(X,A) %>% split(as.numeric(A))
  ch = s %>% lapply(., function(el) chull(el$PC1, el$PC2))
  ch = lapply(names(ch), function(x) s[[x]][ch[[x]],]) %>% do.call(rbind, .) 
  return(ch)
}

draw.obs4 <- function(X,true_cell_cls){
  X = as.data.frame(X2)
  A1 = as.character(repre.v2.0.1$confset[[13]])
  A2 = as.character(repre.v2.0.2$confset[[13]])
  A3 = as.character(repre.v2.0.3$confset[[13]])
  A4 = as.character(repre.v2.0.4$confset[[13]])
  A5 = as.character(repre.v2.0.5$confset[[13]])
  A6 = as.character(repre.v2.0.6$confset[[13]])
  A7 = as.character(repre.v2.0.7$confset[[13]])
  A8 = as.character(repre.v2.0.8$confset[[13]])
  A9 = as.character(repre.v2.0.9$confset[[13]])
  
  cha1 = Ch(X,A1)
  cha2 = Ch(X,A2)
  cha3 = Ch(X,A3)
  cha4 = Ch(X,A4)
  cha5 = Ch(X,A5)
  cha6 = Ch(X,A6)
  cha7 = Ch(X,A7)
  cha8 = Ch(X,A8)
  cha9 = Ch(X,A9)
  
  cha1 = cha1[cha1$A==1,]
  cha2 = cha2[cha2$A==1,]
  cha3 = cha3[cha3$A==1,]
  cha4 = cha4[cha4$A==1,]
  cha5 = cha5[cha5$A==1,]
  cha6 = cha6[cha6$A==1,]
  cha7 = cha7[cha7$A==1,]
  cha8 = cha8[cha8$A==1,]
  cha9 = cha9[cha9$A==1,]
  
  
  ggplot(as.data.frame(X[,1:2]), aes(x=PC1, y=PC2, color = factor(A, level=c("1","0")))) +
    geom_point(shape=19, size=1, alpha=0.5) +
    geom_polygon(data = cha1, aes(fill = A), alpha = 1/5, color = NA) +
    geom_polygon(data = cha2, aes(fill = A), alpha = 0.2/5, color = NA) +
    geom_polygon(data = cha3, aes(fill = A), alpha = 0.3/5, color = NA) +
    geom_polygon(data = cha4, aes(fill = A), alpha = 0.4/5, color = NA) +
    geom_polygon(data = cha5, aes(fill = A), alpha = 0.5/5, color = NA) +
    geom_polygon(data = cha6, aes(fill = A), alpha = 0.6/5, color = NA) +
    geom_polygon(data = cha7, aes(fill = A), alpha = 0.7/5, color = NA) +
    geom_polygon(data = cha8, aes(fill = A), alpha = 0.8/5, color = NA) +
    geom_polygon(data = cha9, aes(fill = A), alpha = 0.9/5, color = NA) +
    guides(color = guide_legend(title="Cluster"), fill=guide_legend(title="Cluster")) +
    theme_classic() +
    theme(legend.position="none")
  
  ggsave("hmmvb_cps.jpg",width=100,height=100,units="mm")
}  

repre.v2$summary
repre.v2$jaccard_mat$V13
########################################################
length(unique(P11))
length(unique(V))
length(unique(repre.p$repre))
length(unique(repre.v$repre))
########################################################
which(true_cell=="2338")==which(P11==4)
which(true_cell=="2339")==which(P11==5)
#which(true_cell=="HL60")==which(P11==1)
which(true_cell=="NPC")==which(P11==10)
which(true_cell=="BJ")==which(P11==6)

which(true_cell=="2338")==which(repre.p$repre==4)
which(true_cell=="2339")==which(repre.p$repre==12)
which(true_cell=="HL60")==which(repre.p$repre==3)
which(true_cell=="NPC")==which(repre.p$repre==1)
#which(true_cell=="BJ")==which(repre.p$repre==8)

which(true_cell=="2338")==which(V==3)
which(true_cell=="2339")==which(V==10)
#which(true_cell=="HL60")==which(V==0)
which(true_cell=="NPC")==which(V==5)
#which(true_cell=="BJ")==which(V==8)
which(true_cell=="K562")==which(V==4)
which(true_cell=="iPS")==which(V==11)

which(true_cell=="2338")==which(repre.v$repre==4)
which(true_cell=="2339")==which(repre.v$repre==18)
#which(true_cell=="HL60")==which(repre.v$repre==1)
which(true_cell=="NPC")==which(repre.v$repre==6)
which(true_cell=="BJ")==which(repre.v$repre==11)
which(true_cell=="K562")==which(repre.v$repre==5)
which(true_cell=="iPS")==which(repre.v$repre==13)
########################################################

n.zb = c(true_cell_cls,P11)-1
n.zb = c(true_cell_cls,repre.p$repre)-1
n.zb = c(true_cell_cls,V+1)-1
n.zb = c(true_cell_cls,repre.v$repre)-1
n.zb = c(true_cell_cls,P11,repre.p$repre,V+1,repre.v$repre)-1
nb=length(n.zb)/300
ta=0.999
setwd("../_package5")
write(n.zb, file = "zb.cls", ncolumns = 1)
system(paste("./labelsw -i zb.cls -o zb.ls -p zb.par -w zb.wt -h zb.h -c zb.summary -b ",nb," -t ",ta," -2",sep=""))
n.dist  <- read.table("zb.par")
n.wt <- read.table("zb.wt")
n.sum <- read.table("zb.summary",nrows=11)
setwd("..")

r.dist = c(1-adjustedRandIndex(true_cell_cls,true_cell_cls),
           1-adjustedRandIndex(true_cell_cls,P11),
           1-adjustedRandIndex(true_cell_cls,repre.p$repre),
           1-adjustedRandIndex(true_cell_cls,V+1),
           1-adjustedRandIndex(true_cell_cls,repre.v$repre))

round(r.dist,4)
round(n.dist[,2],4)

length(unique(repre.p$repre))
length(unique(repre.v$repre))
length(unique(P11))
length(unique(V))

sum(n.sum[,3])
n.wt

n.dist.4 = data.frame(dist=n.dist[2:5,2],
                      method=c("PCAreduce","PCAreduce","HMMVB","HMMVB"),
                      a=c("Whole data","Mean partition","Whole data","Mean partition"))
round(n.dist,4)


library(wesanderson)
library(latex2exp)
bar = ggplot(n.dist.4, aes(x=method,y=dist,fill=factor(a, levels=c("Whole data","Mean partition")))) +
  geom_bar(colour="black", stat="identity", position=position_dodge()) +
  theme_classic() +
  labs(x="",y=TeX('Distance to the truth $^*$')) +
  theme(legend.title=element_blank(),
        legend.position="top") +
  #scale_fill_grey(start = .4, end = .9)
  scale_fill_manual(values=wes_palette(n=2, name="Darjeeling"))
bar

########################################################

library(gridExtra)
g = grid.arrange(
  grobs = list(img.gt,img.p0,img.pm,img.v0,img.vm,bar,
               textGrob("Ground Truth"),textGrob("PCAreduce"),textGrob("HMMVB"),
               textGrob("Whole data",rot=90),textGrob("Mean partition",rot=90),
               textGrob("Dissimilarity from truth")),
  widths = c(1,1),
  heights = c(0.2,1),
  layout_matrix = rbind(c(7,13),
                        c(1,6))
)

g = grid.arrange(
  grobs = list(img.gt,bar,
               textGrob("Ground Truth"),
               textGrob("s")),
  widths = c(1,1),
  heights = c(0.2,1),
  layout_matrix = rbind(c(3,4),
                        c(1,2))
)

ggsave("RD1.jpeg",width=210,height=120,units="mm",g)

########################################################
#### Find overall clustering stability - entropy
########################################################

round(mean(stat.p$dist[,2]),4)
round(mean(stat.v$dist[,2]),4)