% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GenBsSamps.R
\name{GenBsSamps}
\alias{GenBsSamps}
\title{Generates bootstrap samples}
\usage{
GenBsSamps(X, nb, method, stack = T)
}
\arguments{
\item{X}{Original data matrix.}

\item{nb}{The number of ensemble partitions (bootstrap samples) to be generated.}

\item{method}{Baseline clustering methods. User specified function or example methods included in package ("kmeans", "Mclust", "hclust", "dbscan", "PCAreduce", "HMM-VB") can be used. Refer to the Detail.}

\item{stack}{Default is TRUE. The format of the output ensemble partition. If it is TRUE, a stacked vector is returned. If not, a matrix is returned.}
}
\value{
List of ensemble of partitions and detected number of clusters for each partition.
}
\description{
This function generates ensemble of partitions by bootstrap resampling.
}
\examples{
# set OTA package directory and compile C code.
SetOTA(directory)
set.seed(2070)
dat = GenData(n=5000,p=2,C=4)
# set parameters for 'kmeans' method in GenBsSamps.
C=4 
bs.samps = GenBsSamps(dat$X,100,"kmeans")
# find mean partition and uncertainty statistics.
idx = align2(bs.samps$Z,100)
res = MeanPart(bs.samps$Z, 100, idx)
# plot the result on two dimensional space.
plot.part(dat$X,dat$z)          # ground truth
plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
plot.part(dat$X,res$repre)      # OTA
# distance between ground truth and each partition
WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
}
\keyword{ensemble}
