% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WassDist.R
\name{WassDist}
\alias{WassDist}
\title{Wasserstein distance between partitions.}
\usage{
WassDist(Zb.stack, nb)
}
\arguments{
\item{Zb.stack}{Stacked cluster label vector. Reference cluster should be on top of ensemble or partitions. Wasserstein distances of ensemble of partitions to the reference partition is returned.}

\item{nb}{The number of ensemble partitions in Zb.stack.}
}
\value{
Matrix of number of clusters and Wasserstein distances of ensemble of partitions to the reference partition.
}
\description{
This function calculate Wasserstein distance between reference partition and ensemble of partitions.
}
\examples{
# set OTA package directory and compile C code.
SetOTA("/Users/BSS/Documents/7.Research/Cluster_Stability_2/r-package/package5")
set.seed(2070)
dat = GenData(n=5000,p=2,C=4)
# set parameters for 'kmeans' method in GenBsSamps.
C=4 
bs.samps = GenBsSamps(dat$X,100,"kmeans")
# find mean partition and uncertainty statistics.
idx = align2(bs.samps$Z,100)
res = MeanPart(bs.samps$Z, 100, idx)
# plot the result on two dimensional space.
plot.part(dat$X,dat$z)          # ground truth
plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
plot.part(dat$X,res$repre)      # OTA
# distance between ground truth and each partition
WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
}
