% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/align.control.R
\name{align.control}
\alias{align.control}
\title{Align partitions to the reference partition.}
\usage{
align.control(Zb.stack, nb, threshold = 0.8, alpha = 0.1, split = F,
  folder = "align")
}
\arguments{
\item{Zb.stack}{Stacked cluster label vector. Reference cluster should be on top of ensemble or partitions.}

\item{nb}{The number of ensemble partitions in Zb.stack.}

\item{threshold}{Default is 0.8. Threshold for matching statistic. This percentage of overlapping between two clusters is regarded as matching.}

\item{alpha}{Default is 0.1. Confident level for confidence point set.}

\item{split}{Logical. If it is TRUE, split is counted as match in confidence set calculation.}

\item{folder}{Default is "align". The folder name where temporary result files are created.}
}
\value{
List of mean partition(repre), distance of mean partition to each partition in ensemble(dist), output(res), weight matrix(wt), topological stability statistics(summary), confidence point set(confset), cluster alignment and point-based(CAP) separability(jaccard_mat).
}
\description{
This function aligns partitions to the reference partition by optimal transport alignment(OTA).
}
\examples{
# set OTA package directory and compile C code.
SetOTA(directory)
set.seed(2070)
dat = GenData(n=5000,p=2,C=4)
# set parameters for 'kmeans' method in GenBsSamps.
C=4 
bs.samps = GenBsSamps(dat$X,100,"kmeans")
# find mean partition and uncertainty statistics.
idx = align2(bs.samps$Z,100)
res = MeanPart(bs.samps$Z, 100, idx)
# plot the result on two dimensional space.
plot.part(dat$X,dat$z)          # ground truth
plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
plot.part(dat$X,res$repre)      # OTA
# distance between ground truth and each partition
WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
}
