% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/align2.R
\name{align2}
\alias{align2}
\title{Median partition by Optimal Transport Alignment(OTA)}
\usage{
align2(Zb.stack, nb, folder = "align")
}
\arguments{
\item{Zb.stack}{Stacked cluster label vector (n times nb length).}

\item{nb}{The number of ensemble partitions in Zb.stack.}

\item{folder}{Default is "align". The folder name where temporary result files are created.}
}
\value{
Median partition index in ensemble.
}
\description{
This function searches the index of the median partition in ensemble of partitions by optimal transport alignment(OTA).
}
\examples{
# set OTA package directory and compile C code.
SetOTA(directory)
set.seed(2070)
dat = GenData(n=5000,p=2,C=4)
# set parameters for 'kmeans' method in GenBsSamps.
C=4 
bs.samps = GenBsSamps(dat$X,100,"kmeans")
# find mean partition and uncertainty statistics.
idx = align2(bs.samps$Z,100)
res = MeanPart(bs.samps$Z, 100, idx)
# plot the result on two dimensional space.
plot.part(dat$X,dat$z)          # ground truth
plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
plot.part(dat$X,res$repre)      # OTA
# distance between ground truth and each partition
WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
}
