#' Visulize a partition on 2 dimensional space
#'
#' This function plots a partition on 2 dimensional reduced space.
#' @param X Cordinates matrix of data.
#' @param K Cluster labels of data.
#' @return none
#' @examples
#' # set OTA package directory and compile C code.
#' SetOTA(directory)
#' set.seed(2070)
#' dat = GenData(n=5000,p=2,C=4)
#' # set parameters for 'kmeans' method in GenBsSamps.
#' C=4 
#' bs.samps = GenBsSamps(dat$X,100,"kmeans")
#' # find mean partition and uncertainty statistics.
#' idx = align2(bs.samps$Z,100)
#' res = MeanPart(bs.samps$Z, 100, idx)
#' # plot the result on two dimensional space.
#' plot.part(dat$X,dat$z)          # ground truth
#' plot.part(dat$X,kmeans(dat$X,C)$cluster)  # baseline method
#' plot.part(dat$X,res$repre)      # OTA
#' # distance between ground truth and each partition
#' WassDist(c(dat$z,kmeans(dat$X,C)$cluster,res$repre),3)
#' @export


plot_part <- function(X,K,title="",xlab="",ylab=""){
  
  if(dim(X)[2]==2){
    Y = data.frame(PC1=X[,1],PC2=X[,2])
  } else if(dim(X)[2]>2){
    Y = as.data.frame(prcomp(X)$x)
  }
  
  ggplot2::ggplot(Y[,1:2], ggplot2::aes(x=PC1, y=PC2)) +
    ggplot2::geom_point(color=K+1, shape=K, size=1, alpha = 0.5) +
    ggplot2::theme_classic() +
    ggplot2::labs(x=xlab,y=ylab) +
    ggplot2::ggtitle(title) +
    ggplot2::theme(plot.title = ggplot2::element_text(face="bold", hjust = 0.5))
}